/*
<DEPENDENCIES>
    <DEPENDS>Logger</DEPENDS>
    <DEPENDS>Answer</DEPENDS>
    <DEPENDS>Question</DEPENDS>
    <DEPENDS>Evaluation</DEPENDS>
    <DEPENDS>TextAnswer</DEPENDS>
    <DEPENDS>Actions</DEPENDS>
    <DEPENDS>ExerciseHandlerStatus</DEPENDS>
</DEPENDENCIES>
*/
function QeaWorkItem( sId )
{
  if( sId )
  {
    this.init(sId);
  }
}
QeaWorkItem.prototype.init = function( sId )
{
  ;
  this.sId = sId;
  this.bAlreadyStarted = false;
  this.bIsShowSolutionActive = false;
  this.bHasShownSolution = false;
  this.aQuestions = new Array();
  this.nQuestions = 0;
  this.aAnswers = new Array();
  this.nAnswers = 0;
  this.aEvaluations = new Array();
  this.nEvaluations = 0;
  this.aTipStrategies = new Array();
  this.nTipStrategies = 0;
  this.nCorrect = 0;
  this.nWrong = 0;
  this.tipManager = null;
  this.excManager = null;
  this.visibilityBroker = null;
  this.bActive = true;
  this.bCountDynamicAnswersWrong = false;
  this.bEvaluated = false;
  this.bWorkStarted = false;
  this.bHelpApplied = false;
  this.oUserFocusQeaItem = null;
  this.bWorkStopped = false;
  this.bActivated = false;
  this.bWithAlternatives = true;
  this.lastEvaluation = null;
  this.lastUserActivatedEvaluation = null;
  this.oWorkItemBroker = null;
  this.bNotifyView = true;
  this.bNotifyManagers = true;
  this.qeaUndoStack = new QeaUndoStack(this.sId);
  this.offAnswer = new Answer('OffAnswer', 1000);
  this.offQuestion = new Question('OffQuestion', 1000);
}
QeaWorkItem.prototype.getId = function()
{
  return this.sId;
}
QeaWorkItem.prototype.getNumCorrect = function()
{
  return this.nCorrect;
}
QeaWorkItem.prototype.getNumWrong = function()
{
  return this.nWrong;
}
QeaWorkItem.prototype.setWithAlternatives = function( bWithAlternatives )
{
  ;
  this.bWithAlternatives = bWithAlternatives;
}
QeaWorkItem.prototype.setAnswerBasedEvaluation = function( bAnswerBasedEvaluation )
{
  ;
  this.bAnswerBasedEvaluation = bAnswerBasedEvaluation;
}
QeaWorkItem.prototype.setWorkItemBroker = function( oWorkItemBroker )
{
  ;
  this.oWorkItemBroker = oWorkItemBroker;
  this.oWorkItemBroker.addResetHandle(this.getResetAction());
  this.oWorkItemBroker.addEvaluateHandle(this.getEvaluateAction());
  this.oWorkItemBroker.addShowSolutionHandle(this.getShowSolutionAction());
  this.oWorkItemBroker.addHideSolutionHandle(this.getHideSolutionAction());
  this.oWorkItemBroker.addNoticedHandle(this.getNoticedAction());
}
QeaWorkItem.prototype.registerQuestion = function( question )
{
  ;
  if( this.aQuestions[question.getId()] != null )
  {
    ;
  }
  else
  {
    this.aQuestions[question.getId()] = question;
    this.nQuestions++;
    ;
    question.setWorkItem(this);
  }
}
QeaWorkItem.prototype.registerAnswer = function( answer )
{
  ;
  if( this.aAnswers[answer.getId()] != null )
  {
    ;
  }
  else
  {
    this.aAnswers[answer.getId()] = answer;
    this.nAnswers++;
    ;
    answer.setWorkItem(this);
  }
}
QeaWorkItem.prototype.registerEvaluation = function( evaluation )
{
  ;
  if( this.aEvaluations[evaluation.getId()] != null )
  {
    ;
  }
  else
  {
    this.aEvaluations[evaluation.getId()] = evaluation;
    this.nEvaluations++;
    ;
    evaluation.setWorkItem(this);
  }
  if( evaluation.isSolution() && !(evaluation.getAnswer() == this.offAnswer || evaluation.getQuestion()
          == this.offQuestion ) )
  {
    this.nCorrect++;
  }
  if( evaluation.isCorrect() && !(evaluation.getAnswer() == this.offAnswer || evaluation.getQuestion()
          == this.offQuestion ) )
  {
    evaluation.getAnswer().incCorrectEvaluations();
    evaluation.getQuestion().incCorrectEvaluations();
  }
}
QeaWorkItem.prototype.registerTipStrategy = function( tipStrategy )
{
  ;
  if( this.aTipStrategies[tipStrategy.getId()] != null )
  {
    ;
  }
  else
  {
    this.aTipStrategies[tipStrategy.getId()] = tipStrategy;
    this.nTipStrategies++;
    ;
    tipStrategy.setWorkItem(this);
  }
}
QeaWorkItem.prototype.setTipManager = function( tipManager )
{
  ;
  this.tipManager = tipManager;
}
QeaWorkItem.prototype.setExerciseManager = function( excManager )
{
  ;
  this.excManager = excManager;
}
QeaWorkItem.prototype.setVisibilityBroker = function( broker )
{
  ;
  this.visibilityBroker = broker;
}
QeaWorkItem.prototype.setActive = function( bActive )
{
  ;
  this.bActive = bActive;
  if( this.tipManager ) this.tipManager.changed(this);
}
QeaWorkItem.prototype.getActivateAction = function()
{
  return new ObjectMethodAction(this, 'setActive', true);
}
QeaWorkItem.prototype.getDeactivateAction = function()
{
  return new ObjectMethodAction(this, 'setActive', false);
}
QeaWorkItem.prototype.start = function()
{
  ;
  if( !this.bAlreadyStarted )
  {
    this.bAlreadyStarted = true;
    if( this.bAnswerBasedEvaluation )
      this.nWrong = this.nAnswers;
    else
      this.nWrong = this.calculateWrong();
    this.offAnswer.incCorrectEvaluations();
    this.registerAnswer(this.offAnswer);
    this.offAnswer.start();
    this.offQuestion.incCorrectEvaluations();
    this.registerQuestion(this.offQuestion);
    this.offQuestion.start();
    this.qeaUndoStack.lock();
    this.setNotifyView(false);
    for( var id in this.aQuestions )
    {
      if( this.aQuestions[id].countCorrectEvaluations() == 0 )
      {
        ;
        evaluation = new Evaluation(this.sId + '_' + this.aQuestions[id].getId() + '_'
                + this.offAnswer.getId(), this.aQuestions[id], this.offAnswer, true, true);
        this.registerEvaluation(evaluation);
        evaluation.setActive(true, 'system_off');
      }
    }
    for( var id in this.aAnswers )
    {
      if( this.aAnswers[id].countCorrectEvaluations() == 0 )
      {
        ;
        evaluation = new Evaluation(this.sId + '_' + this.offQuestion.getId() + '_'
                + this.aAnswers[id].getId(), this.offQuestion, this.aAnswers[id], true, true);
        this.registerEvaluation(evaluation);
        evaluation.setActive(true, 'system_off');
      }
    }
    this.setNotifyView(true);
    this.qeaUndoStack.unlock();
  }
}
QeaWorkItem.prototype.makeFakedUserActivity = function()
{
  this.workStarted();
  if( this.bNotifyManagers )
  {
    var excStatus = new ExerciseHandlerStatus(this.nCorrect, 0, this.nCorrect, 0, 0, 0, this);
    if( this.excManager ) this.excManager.solved(this, excStatus);
    if( this.tipManager ) this.tipManager.changed(this);
    if( this.oWorkItemBroker )
    {
      this.oWorkItemBroker.solved(excStatus);
      if( !this.bHelpApplied && (excStatus.nCorrectSystem || excStatus.nWrongSystem) )
      {
        this.bHelpApplied = true;
        this.oWorkItemBroker.doHelpApplied();
      }
    }
  }
}
QeaWorkItem.prototype.hasShowTip = function( bShow, bStepByStep, bSolutable, bCycleTips )
{
  if( bShow )
  {
    ;
  }
  var bHasTip = false;
  for( var id in this.aTipStrategies )
  {
    var tipStrategy = this.aTipStrategies[id];
    if( tipStrategy )
    {
      ;
      bHasTip = tipStrategy.hasShowTip(bShow, bStepByStep, bSolutable, bCycleTips);
      if( bHasTip )
      {
        ;
        break;
      }
    }
  }
  if( !bHasTip )
  {
    ;
  }
  return bHasTip;
}
QeaWorkItem.prototype.hasTip = function( bStepByStep, bSolutable, bCycleTips )
{
  ;
  if( this.bActive )
  {
    return this.hasShowTip(false, bStepByStep, bSolutable, bCycleTips);
  }
  else
  {
    return false;
  }
}
QeaWorkItem.prototype.showTip = function( bStepByStep, bSolutable, bCycleTips )
{
  ;
  var bHasTip = false;
  if( this.bActive )
  {
    var bLocalCycleTips = bCycleTips;
    bHasTip = true;
    if( !bStepByStep )
    {
      bLocalCycleTips = false;
    }
    while( bHasTip )
    {
      bHasTip = this.hasShowTip(true, bStepByStep, bSolutable, bLocalCycleTips);
      this.bUserAction = false;
      if( bStepByStep )
      {
        break;
      }
    }
  }
  return bHasTip;
}
QeaWorkItem.prototype.assign = function( question, answer )
{
  ;
  var bAssigned = false;
  var evaluation = this.getOrCreateEvaluation(question, answer);
  if( evaluation != null )
  {
    if( !evaluation.isActive() )
    {
      evaluation.setActive(true);
    }
    bAssigned = evaluation.isActive();
  }
  else
  {
    ;
  }
  return bAssigned;
}
QeaWorkItem.prototype.assignOrConfirmByText = function( question, sText )
{
  this.assignByText(question, sText, true);
}
QeaWorkItem.prototype.assignByText = function( question, sText, bConfirmActive )
{
  ;
  var bAssigned = false;
  var answer = this.getOrCreateAnswerByText(sText);
  if( answer != null )
  {
    var evaluation = this.getOrCreateEvaluation(question, answer);
    if( evaluation != null )
    {
      if( !evaluation.isActive() || bConfirmActive )
      {
        evaluation.setActive(true);
      }
      bAssigned = evaluation.isActive();
    }
    else
    {
      ;
    }
  }
  else
  {
    ;
  }
  return bAssigned;
}
QeaWorkItem.prototype.unassign = function( question, answer )
{
  ;
  var bUnassigned = false;
  var evaluation = this.getEvaluation(question, answer);
  if( evaluation != null )
  {
    if( evaluation.isActive() )
    {
      evaluation.setActive(false);
    }
    bUnassigned = !evaluation.isActive();
  }
  else
  {
    ;
    bUnassigned = true;
  }
  return bUnassigned;
}
QeaWorkItem.prototype.getOrCreateEvaluation = function( question, answer )
{
  ;
  var evaluation = this.getEvaluation(question, answer);
  if( evaluation == null )
  {
    ;
    evaluation = new Evaluation(this.sId + '_' + question.getId() + '_'
            + answer.getId(), question, answer, false, false);
    if( evaluation != null )
    {
      this.registerEvaluation(evaluation);
    }
  }
  return evaluation;
}
QeaWorkItem.prototype.getEvaluation = function( question, answer )
{
  ;
  var evaluation = null;
  for( var id in this.aEvaluations )
  {
    if( this.aEvaluations[id].getQuestion() == question && this.aEvaluations[id].getAnswer() == answer )
    {
      evaluation = this.aEvaluations[id];
      break;
    }
  }
  return evaluation;
}
QeaWorkItem.prototype.getEvaluations = function( question, answer )
{
  ;
  var evaluations = new Array();
  for( var id in this.aEvaluations )
  {
    if( this.aEvaluations[id].getQuestion() == question && this.aEvaluations[id].getAnswer() == answer )
    {
      evaluations.push(this.aEvaluations[id]);
    }
  }
  return evaluations;
}
QeaWorkItem.prototype.getOrCreateAnswerByText = function( sText )
{
  ;
  var answer = this.getAnswerByText(sText);
  if( answer == null )
  {
    ;
    answer = new TextAnswer(this.sId + '_' + sText, sText, 1000);
    if( answer != null )
    {
      this.registerAnswer(answer);
    }
  }
  return answer;
}
QeaWorkItem.prototype.getAnswerByText = function( sText )
{
  ;
  var answer = null;
  for( var id in this.aAnswers )
  {
    if( this.aAnswers[id].match(sText) )
    {
      answer = this.aAnswers[id];
      break;
    }
  }
  return answer;
}
QeaWorkItem.prototype.setUserFocus = function( oUserFocusQeaItem )
{
  ;
  if( this.oUserFocusQeaItem )
  {
    this.oUserFocusQeaItem.bUserFocus = false;
  }
  this.oUserFocusQeaItem = oUserFocusQeaItem;
  if( this.oUserFocusQeaItem )
  {
    this.oUserFocusQeaItem.bUserFocus = true;
  }
}
QeaWorkItem.prototype.isCurrentWorkItem = function()
{
  if( this.excManager )
  {
    var lastWorkItem = this.excManager.currentExecHandler;
    return (lastWorkItem == this);
  }
  else
  {
    return false;
  }
}
QeaWorkItem.prototype.getOrCreateAnswerByTextAndGridPos = function( sText, nX, nY )
{
  ;
  var answer = this.getAnswerByTextAndGridPos(sText, nX, nY);
  if( answer == null )
  {
    ;
    answer = new TextGridAnswer('TextGridAnswer_' + this.sId + sText + '_X_' + nX + '_Y_' + nY, 1000, nX, nY, sText);
    if( answer != null )
    {
      this.registerAnswer(answer);
    }
  }
  return answer;
}
QeaWorkItem.prototype.getAnswerByTextAndGridPos = function( sText, nX, nY )
{
  ;
  var answer = null;
  for( var id in this.aAnswers )
  {
    if( this.aAnswers[id].match(sText, nX, nY) )
    {
      answer = this.aAnswers[id];
      break;
    }
  }
  return answer;
}
QeaWorkItem.prototype.getOrCreateAnswerByPosAndLength = function( nStartPos, nLength )
{
  ;
  var answer = this.getAnswerByPosAndLength(nStartPos, nLength);
  if( answer == null )
  {
    ;
    answer = new HighlighterAnswer('HighlighterAnswer_' + this.sId + '_Start_' + nStartPos + '_length_'
            + nLength, nStartPos, nLength, 1000);
    if( answer != null )
    {
      this.registerAnswer(answer);
    }
  }
  return answer;
}
QeaWorkItem.prototype.getAnswerByPosAndLength = function( nStartPos, nLength )
{
  ;
  var answer = null;
  for( var id in this.aAnswers )
  {
    if( this.aAnswers[id].match(nStartPos, nLength) )
    {
      answer = this.aAnswers[id];
      break;
    }
  }
  return answer;
}
QeaWorkItem.prototype.willChange = function( evaluation )
{
  ;
  this.qeaUndoStack.push(new QeaUndoItem(evaluation.getId(), evaluation));
}
QeaWorkItem.prototype.setUndoBreak = function( sTag )
{
  ;
  this.qeaUndoStack.push(new QeaUndoItem(sTag, null));
}
QeaWorkItem.prototype.undo = function( bRestoreSystemContribution, sTag )
{
  ;
  var sLastId = null;
  this.qeaUndoStack.lock();
  var bDone = false;
  while( !bDone )
  {
    var undoItem = this.qeaUndoStack.pop();
    if( undoItem != null )
    {
      sLastId = undoItem.getId();
      if( undoItem.evaluation )
      {
        undoItem.evaluation.setActive(undoItem.bActive, undoItem.sOrigin);
        if( bRestoreSystemContribution )
        {
          undoItem.evaluation.setSystemConstuction(undoItem.bSystemConstructive);
          undoItem.evaluation.setSystemCorrection(undoItem.bSystemCorrective);
          undoItem.evaluation.setSystemPartialConstuction(undoItem.bSystemPartialConstructive);
          undoItem.evaluation.setSystemPartialCorrection(undoItem.bSystemPartialCorrective);
        }
      }
    }
    else
    {
      ;
      bDone = true;
    }
    if( sTag )
    {
      if( sTag == sLastId )
      {
        bDone = true;
      }
    }
    else
    {
      bDone = true;
    }
  }
  this.qeaUndoStack.unlock();
  return sLastId;
}
QeaWorkItem.prototype.changed = function( evaluation )
{
  ;
  if( this.bNotifyManagers )
  {
    this.lastEvaluation = evaluation;
    if( evaluation )
    {
      switch( evaluation.getOrigin() )
              {
        case 'user':
          this.lastUserActivatedEvaluation = evaluation;
          break;
      }
      if( evaluation.isActive() )
      {
        switch( evaluation.getOrigin() )
                {
          case 'system_set':
            this.lastUserActivatedEvaluation = evaluation;
            break;
        }
      }
    }
    if( !this.bWorkStarted && !this.bEvaluated )
    {
      if( this.bHasShownSolution )
      {
        this.bHasShownSolution = false;
      }
      else
      {
        if( evaluation )
        {
          switch( evaluation.getOrigin() )
                  {
            case 'system_solution':
              this.noticed();
              break;
            case 'user':
            case 'system_set':
            case 'system_approve':
            case 'system_brand':
              this.workStarted();
              break;
          }
        }
      }
    }
    this.propagateStatus(evaluation);
    if( evaluation && evaluation.getOrigin() == 'user' )
    {
      this.qeaUndoStack.flush();
    }
  }
}
QeaWorkItem.prototype.workStarted = function()
{
  if( !this.bWorkStarted )
  {
    this.noticed();
    this.bWorkStarted = true;
    if( this.oWorkItemBroker )
    {
      this.oWorkItemBroker.doWorkStarted();
    }
    this.excManager.workItemStarted(this);
  }
}
QeaWorkItem.prototype.propagateStatus = function( evaluation )
{
  ;
  if( this.bNotifyManagers )
  {
    var excStatus = this.getExerciseStatus();
    ;
    if( this.excManager ) this.excManager.solved(this, excStatus);
    if( this.tipManager ) this.tipManager.changed(this);
    if( this.oWorkItemBroker )
    {
      this.oWorkItemBroker.solved(excStatus);
      if( !this.bHelpApplied && (excStatus.nCorrectSystem || excStatus.nWrongSystem) )
      {
        this.bHelpApplied = true;
        this.oWorkItemBroker.doHelpApplied();
      }
    }
    if( evaluation && evaluation.getOrigin() == 'system_solution' )
    {
      if( this.visibilityBroker ) this.visibilityBroker.activity();
    }
  }
}
QeaWorkItem.prototype.calculateWrong = function()
{
  var questionAssignments = 0;
  var answerAssignments = 0;
  for( var id in this.aQuestions )
  {
    questionAssignments += this.aQuestions[id].getMax();
  }
  for( var id in this.aAnswers )
  {
    answerAssignments += this.aAnswers[id].getMax();
  }
  var assignments = (questionAssignments < answerAssignments) ? questionAssignments : answerAssignments;
  return (assignments < (this.nAnswers * this.nQuestions)) ? assignments : (this.nAnswers * this.nQuestions);
}
QeaWorkItem.prototype.getExerciseStatus = function()
{
  ;
  if( this.bCountDynamicAnswersWrong )
  {
    this.nWrong = this.calculateWrong();
  }
  var nCorrectStatus = 0;
  var nWrongStatus = 0;
  var nCorrectSystem = 0;
  var nWrongSystem = 0;
  for( var id in this.aEvaluations )
  {
    var e = this.aEvaluations[id];
    if( e.isActive() )
    {
      if( e.isCorrect() )
      {
        if( !e.hasOffItem() )
        {
          nCorrectStatus++;
        }
      }
      else
      {
        nWrongStatus++;
      }
    }
    else
    {
      if( !this.bWithAlternatives && e.isCorrect() )
      {
        if( !e.hasOffItem() )
        {
          nWrongStatus++;
        }
      }
    }
    if( e.hasSystemConstuction() || e.hasSystemPartialConstuction() )
    {
      nCorrectSystem++;
    }
    if( e.hasSystemCorrection() || e.hasSystemPartialCorrection() )
    {
      nWrongSystem++;
    }
  }
  nWrongStatus = nWrongStatus > this.nWrong ? this.nWrong : nWrongStatus;
  nWrongSystem = nWrongSystem > this.nWrong ? this.nWrong : nWrongSystem;
  var sOrigin = undefined;
  if( this.lastEvaluation && this.lastEvaluation.sOrigin )
  {
    sOrigin = this.lastEvaluation.sOrigin;
  }
  return new ExerciseHandlerStatus(this.nCorrect, this.nWrong, nCorrectStatus, nWrongStatus, nCorrectSystem, nWrongSystem, this, sOrigin);
}
QeaWorkItem.prototype.evaluate = function()
{
  ;
  this.workStarted();
  this.propagateStatus();
  this.bEvaluated = true;
  this.bNotifyManagers = false;
  for( var id in this.aEvaluations )
  {
    this.aEvaluations[id].evaluate();
  }
  for( var id in this.aQuestions )
  {
    this.aQuestions[id].evaluate();
  }
  for( var id in this.aAnswers )
  {
    this.aAnswers[id].evaluate();
  }
  var excStatus = this.getExerciseStatus();
  if( excStatus.nCorrect == excStatus.nCorrectStatus && excStatus.nWrongStatus == 0 )
  {
  }
  else
  {
    if( this.visibilityBroker )
      this.visibilityBroker.evaluatesWrong();
  }
  this.bNotifyManagers = true;
  this.workStopped();
}
QeaWorkItem.prototype.reset = function()
{
  ;
  if( this.oWorkItemBroker && this.bWorkStarted && !this.bWorkStopped ) 
  {
    this.oWorkItemBroker.doWorkAborted();
  }
  this.excManager.workItemAborted(this);
  this.bNotifyManagers = false;
  this.bNotifyView = true;
  this.bEvaluated = false;
  this.bActivated = false;
  this.bIsShowSolutionActive = false;
  this.bHasShownSolution = false;
  for( var id in this.aQuestions )
  {
    this.aQuestions[id].reset();
  }
  for( var id in this.aAnswers )
  {
    this.aAnswers[id].reset();
  }
  for( var id in this.aEvaluations )
  {
    this.aEvaluations[id].reset();
  }
  for( var id in this.aEvaluations )
  {
    if( this.aEvaluations[id].bIsInitialActive )
    {
      this.aEvaluations[id].setActive(true, 'system_off');
    }
  }
  for( var id in this.aQuestions )
  {
    if( this.aQuestions[id].countCorrectEvaluations() == 0 )
    {
      evaluation = this.getEvaluation(this.aQuestions[id], this.offAnswer);
      if( evaluation )
      {
        evaluation.setActive(true, 'system_off');
      }
    }
  }
  for( var id in this.aAnswers )
  {
    if( this.aAnswers[id].countCorrectEvaluations() == 0 )
    {
      evaluation = this.getEvaluation(this.offQuestion, this.aAnswers[id]);
      if( evaluation )
      {
        evaluation.setActive(true, 'system_off');
      }
    }
  }
  this.bNotifyManagers = true;
  this.qeaUndoStack.flush();
  this.bWorkStarted = false;
  this.bHelpApplied = false;
  this.bWorkStopped = false;
  this.bActive = true;
  this.propagateStatus();
}
QeaWorkItem.prototype.workStopped = function()
{
  ;
  if( this.oWorkItemBroker && ! this.bWorkStopped )
  {
    this.oWorkItemBroker.doWorkStopped();
  }
  this.excManager.workItemStopped(this);
  this.bActive = false;
  this.bWorkStopped = true;
  this.propagateStatus();
}
QeaWorkItem.prototype.isUncovered = function()
{
  ;
  var bIsUncovered = false;
  for( var id in this.aEvaluations )
  {
    this.aEvaluations[id].setUncovered(false);
  }
  for( var id in this.aQuestions )
  {
    this.aQuestions[id].setUncovered(false);
  }
  for( var id in this.aAnswers )
  {
    this.aAnswers[id].setUncovered(false);
  }
  for( var id in this.aEvaluations )
  {
    this.aEvaluations[id].checkUncovered();
  }
  var nUncoveredQuestions = 0;
  for( var id in this.aQuestions )
  {
    if( this.aQuestions[id].isUncovered() )
    {
      nUncoveredQuestions++;
    }
  }
  if( nUncoveredQuestions == this.nQuestions - 1 )
  {
    bIsUncovered = true;
    ;
  }
  if( !bIsUncovered )
  {
    var nUncoveredAnswers = 0;
    for( var id in this.aAnswers )
    {
      if( this.aAnswers[id].isUncovered() )
      {
        nUncoveredAnswers++;
      }
    }
    if( nUncoveredAnswers == this.nAnswers - 1 )
    {
      bIsUncovered = true;
      ;
    }
  }
  if( !bIsUncovered )
  {
    var nCovered = 0;
    for( var id in this.aEvaluations )
    {
      if( this.aEvaluations[id].isUncovered() )
      {
        ;
      }
      else
      {
        nCovered++;
        ;
      }
    }
    ;
    bIsUncovered = nCovered == 0;
  }
  return bIsUncovered;
}
QeaWorkItem.prototype.showSolution = function()
{
  ;
  this.bIsShowSolutionActive = true;
  this.setUndoBreak('system_solution_sentinel');
  this.bNotifyManagers = false;
  var aInactiveEvaluations = new Array();
  for( var id in this.aEvaluations )
  {
    var e = this.aEvaluations[id];
    e.bEvaluated = false;
    if( !e.isSolution() )
    {
      if( e.isActive() )
      {
        e.setActive(false, 'system_solution');
      }
      else
      {
        aInactiveEvaluations[id] = e;
      }
    }
  }
  for( var id in this.aEvaluations )
  {
    var e = this.aEvaluations[id];
    if( e.isSolution() )
    {
      e.setActive(true, 'system_solution');
    }
  }
  for( var id in aInactiveEvaluations )
  {
    var e = aInactiveEvaluations[id];
    e.setActive(false, 'system_solution');
  }
  this.changed();
}
QeaWorkItem.prototype.hideSolution = function()
{
  ;
  if( this.bIsShowSolutionActive )
  {
    this.bIsShowSolutionActive = false;
    this.bHasShownSolution = true;
  }
  this.bNotifyManagers = false;
  var bRestoreSystemContribution = false;
  var sentinel = this.undo(bRestoreSystemContribution, 'system_solution_sentinel');
  if( sentinel != 'system_solution_sentinel' )
  {
    ;
  }
  if( this.bEvaluated )
  {
    this.evaluate();
  }
  this.bNotifyManagers = true;
  this.changed();
}
QeaWorkItem.prototype.dumpStatus = function()
{
  ;
  ;
  for( var id in this.aEvaluations )
  {
    var e = this.aEvaluations[id];
    ;
  }
  this.isUncovered();
  var nCovered = 0;
  for( var id in this.aEvaluations )
  {
    if( this.aEvaluations[id].isUncovered() )
    {
      ;
    }
    else
    {
      nCovered++;
      ;
    }
  }
  ;
}
QeaWorkItem.prototype.noticed = function()
{
  if( !this.bActivated )
  {
    this.bActivated = true;
    if( this.oWorkItemBroker )
    {
      this.oWorkItemBroker.activated();
    }
    this.excManager.workItemActivated(this);
  }
}
QeaWorkItem.prototype.setNotifyView = function( bNotifyView )
{
  //;
  this.bNotifyView = bNotifyView;
}
QeaWorkItem.prototype.getNotifyView = function()
{
  //;
  return this.bNotifyView;
}
QeaWorkItem.prototype.setNotifyManagers = function( bNotifyManagers )
{
  //;
  this.bNotifyManagers = bNotifyManagers;
}
QeaWorkItem.prototype.getNotifyManagers = function()
{
  //;
  return this.bNotifyManagers;
}
QeaWorkItem.prototype.setCountDynamicAnswersWrong = function( bCountDynamicAnswersWrong )
{
  //;
  this.bCountDynamicAnswersWrong = bCountDynamicAnswersWrong;
}
QeaWorkItem.prototype.getResetAction = function()
{
  return new ObjectMethodAction(this, "reset");
}
QeaWorkItem.prototype.getEvaluateAction = function()
{
  return new ObjectMethodAction(this, "evaluate");
}
QeaWorkItem.prototype.getShowSolutionAction = function()
{
  return new ObjectMethodAction(this, "showSolution");
}
QeaWorkItem.prototype.getHideSolutionAction = function()
{
  return new ObjectMethodAction(this, "hideSolution");
}
QeaWorkItem.prototype.getNoticedAction = function()
{
  return new ObjectMethodAction(this, "noticed");
}
function QeaUndoItem( sId, evaluation )
{
  if( sId )
  {
    this.init(sId, evaluation);
  }
}
QeaUndoItem.prototype.init = function( sId, evaluation )
{
  ;
  this.sId = sId;
  this.evaluation = evaluation;
  if( evaluation )
  {
    this.bActive = evaluation.isActive();
    this.sOrigin = evaluation.getOrigin();
    this.bSystemConstructive = evaluation.hasSystemConstuction();
    this.bSystemCorrective = evaluation.hasSystemCorrection();
    this.bSystemPartialConstructive = evaluation.hasSystemPartialConstuction();
    this.bSystemPartialCorrective = evaluation.hasSystemPartialCorrection();
  }
  else
  {
    this.bActive = false;
    this.sOrigin = '';
    this.bSystemConstructive = false;
    this.bSystemCorrective = false;
    this.bSystemPartialConstructive = false;
    this.bSystemPartialCorrective = false;
    this.bSystemPartialConstructive = false;
    this.bSystemPartialCorrective = false;
  }
}
QeaUndoItem.prototype.getId = function()
{
  return this.sId;
}
function QeaUndoStack( sId )
{
  if( sId )
  {
    this.init(sId);
  }
}
QeaUndoStack.prototype.init = function( sId )
{
  ;
  this.sId = sId;
  this.bLock = false;
  this.aUndoItems = new Array();
}
QeaUndoStack.prototype.getId = function()
{
  return this.sId;
}
QeaUndoStack.prototype.hasUndoSentinel = function( sTag )
{
  ;
  var bFound = false;
  for( var i = 0; i < this.aUndoItems.length; i++ )
  {
    if( sTag == this.aUndoItems[i].getId() )
    {
      bFound = true;
      break;
    }
  }
  return bFound;
}
QeaUndoStack.prototype.push = function( undoItem )
{
  ;
  if( !this.bLock )
  {
    this.aUndoItems.push(undoItem);
  }
}
QeaUndoStack.prototype.pop = function()
{
  ;
  return this.aUndoItems.pop();
}
QeaUndoStack.prototype.flush = function()
{
  ;
  if( !this.bLock )
  {
    this.aUndoItems = new Array();
  }
}
QeaUndoStack.prototype.lengh = function()
{
  //;
  return this.aUndoItems.length;
}
QeaUndoStack.prototype.isLocked = function()
{
  //;
  return this.bLock;
}
QeaUndoStack.prototype.lock = function()
{
  ;
  this.bLock = true;
}
QeaUndoStack.prototype.unlock = function()
{
  ;
  this.bLock = false;
}
QeaUndoStack.prototype.setLock = function( bLock )
{
  ;
  this.bLock = bLock;
}
