/*
<DEPENDENCIES>
    <DEPENDS>Logger</DEPENDS>
    <DEPENDS>Browser</DEPENDS>
    <DEPENDS>Actions</DEPENDS>
    <DEPENDS>SoundBroker</DEPENDS>
    <DEPENDS>SpeechRecognitionBroker</DEPENDS>
    <DEPENDS>SpeechRecognitionControlBroker</DEPENDS>
    <DEPENDS>ObjectMethodAction</DEPENDS>
    <DEPENDS>DialogStoryRowSystem</DEPENDS>
    <DEPENDS>DialogStoryRowUser</DEPENDS>
</DEPENDENCIES>
*/
DialogStoryManager_instances = new Array();
function DialogStoryManager_getInstance(sId)
{
  return DialogStoryManager_instances[sId];
}
function DialogStoryManager(sId, nConfidenceLevel, oStartStoryElementSystem, containerDomId)
{
  DialogStoryManager_instances[sId] = this;
  this.init(sId, nConfidenceLevel, oStartStoryElementSystem, containerDomId);
}
DialogStoryManager.prototype.init = function(sId, nConfidenceLevel, oStartStoryElementSystem, containerDomId)
{
  this.sId = sId;
  this.nConfidenceLevel = nConfidenceLevel;
  this.oStartStoryElementSystem = oStartStoryElementSystem;
  this.containerDomId = containerDomId;
  this.containerElement = Browser.getElementById(containerDomId);
  this.scrollBroker = null;
  this.dialogStoryBroker = null;
  this.speechRecognizer = null;
  this.speechRecognitionControlBroker = null;
  this.speechRecognitionBroker = null;
  this.soundPlayerRegistryManager = null;
  this.view = new Object();
  this.view.sFgColorHighlight = null;
  this.view.sBgColorHighlight = null;
  this.view.sTextStyleClass = null;
  this.view.sSystemRowStyleClass = null;
  this.view.sUserRowStyleClass = null;
  this.view.bUseFeedbackPanel = false;
  this.view.feedbackPanelDomId = null;
  this.view.feedbackPanelDivElement = null;
  this.view.nFeedbackTimeoutSeconds = 2;
  this.view.sFeedbackDefaultText = '';
  this.view.bShowSoundPlayer = false;
  this.view.sSoundPlayerNormalImageUrl = null;
  this.view.sSoundPlayerHighlightImageUrl = null;
  this.view.nSoundPlayerImageWidth = 0;
  this.view.nSoundPlayerImageHeight= 0;
  this.view.displayElement = this.containerElement;
  this.bStoryStarted = false;
  this.bStoryRunning = false;
  this.bSoundPlaybackPhase = false;
  this.bRecognizeUserInputPhase = false;
  this.currentState = null;
}
DialogStoryManager.prototype.setStyle = function(
                  sFgColorHighlight, sBgColorHighlight, sTextStyleClass,
                  sUserRowStyleClass, sSystemRowStyleClass)
{
  this.view.sFgColorHighlight = sFgColorHighlight;
  this.view.sBgColorHighlight = sBgColorHighlight;
  this.view.sTextStyleClass = sTextStyleClass;
  this.view.sUserRowStyleClass = sUserRowStyleClass;
  this.view.sSystemRowStyleClass = sSystemRowStyleClass;
}
DialogStoryManager.prototype.setFeedbackPanel = function(divisionDomId, feedbackTimeoutSeconds)
{
  this.view.bUseFeedbackPanel = true;
  this.view.feedbackPanelDomId = divisionDomId;
  if(feedbackTimeoutSeconds)
  {
    this.view.nFeedbackTimeoutSeconds = feedbackTimeoutSeconds;
  }
}
DialogStoryManager.prototype.setDefaultFeedback = function(sDefaultFeedback)
{
  this.view.sFeedbackDefaultText = sDefaultFeedback;
}
DialogStoryManager.prototype.setScrollBroker = function(scrollBroker)
{
  this.scrollBroker = scrollBroker;
}
DialogStoryManager.prototype.setDialogStoryBroker = function(dialogStoryBroker)
{
  this.dialogStoryBroker = dialogStoryBroker;
  this.dialogStoryBroker.addRequestStartHandle(new ObjectMethodAction(this, 'startStory'));
  this.dialogStoryBroker.addRequestStopHandle(new ObjectMethodAction(this, 'stopStory'));
  this.dialogStoryBroker.addRequestContinueHandle(new ObjectMethodAction(this, 'continueStory'));
}
DialogStoryManager.prototype.setSpeechRecognizer = function(speechRecognizer)
{
  this.speechRecognizer = speechRecognizer;
  this.speechRecognitionBroker = new SpeechRecognitionBroker();
  this.speechRecognizer.setRecognitionBroker(this.speechRecognitionBroker);
  this.speechRecognitionControlBroker = new SpeechRecognitionControlBroker();
  this.speechRecognizer.setControlBroker(this.speechRecognitionControlBroker);
  this.speechRecognitionBroker.addEarDataRecognizedHandle(new ObjectMethodAction(this, 'somethingRecognized'));
  this.speechRecognitionControlBroker.addRecognitionStoppedHandle(new ObjectMethodAction(this, 'recognitionStopped'));
}
DialogStoryManager.prototype.setSoundPlayerRegistryManager = function(soundPlayerRegistryManager)
{
  this.soundPlayerRegistryManager = soundPlayerRegistryManager;
}
DialogStoryManager.prototype.setSoundPlayerImages = function(
      sSoundPlayerNormalImageUrl, sSoundPlayerHighlightImageUrl,
      nSoundPlayerImageWidth, nSoundPlayerImageHeight)
{
  this.view.bShowSoundPlayer = true;
  this.view.sSoundPlayerNormalImageUrl = sSoundPlayerNormalImageUrl;
  this.view.sSoundPlayerHighlightImageUrl = sSoundPlayerHighlightImageUrl;
  this.view.nSoundPlayerImageWidth = nSoundPlayerImageWidth;
  this.view.nSoundPlayerImageHeight = nSoundPlayerImageHeight;
}
DialogStoryManager.prototype.getViewConfig = function()
{
  return this.view;
}
DialogStoryManager.prototype.start = function()
{
  if(this.view.feedbackPanelDomId)
  {
    this.view.feedbackPanelDivElement = Browser.getElementById(this.view.feedbackPanelDomId);
    this.clearFeedback();
  }
}
DialogStoryManager.prototype.startStory = function()
{
  ;
  if(!this.bStoryStarted && !this.bStoryRunning)
  {
    ;
    this.currentState = new StoryStateSystem(this, this.oStartStoryElementSystem);
    this.currentState.run();
    this.bStoryStarted = true;
    this.bStoryRunning = true;
  }
  if(this.dialogStoryBroker)
  {
    this.dialogStoryBroker.started();
  }
}
DialogStoryManager.prototype.stopStory = function()
{
  ;
  if(this.bStoryRunning)
  {
    this.stopRecognition();
    this.bStoryRunning = false;
    if(this.currentState)
    {
      this.currentState.stop();
    }
    ;
    this.currentState = null;
  }
  if(this.dialogStoryBroker)
  {
    this.dialogStoryBroker.stopped();
  }
}
DialogStoryManager.prototype.continueStory = function()
{
  ;
  this.clearFeedback();
  if(this.currentState)
  {
    this.currentState.continueStory();
  }
}
DialogStoryManager.prototype.resetStory = function()
{
  ;
  this.stopStory();
  if(this.bStoryStarted)
  {
    this.bStoryStarted = false;
    this.clearFeedback();
    this.view.displayElement.innerHTML = '';
  }
}
DialogStoryManager.prototype.someSoundStopped = function()
{
  ;
  if(this.bRecognizeUserInputPhase && this.bStoryRunning)
  {
    this.bSoundPlaybackPhase = false;
    this.startRecognition();
  }
}
DialogStoryManager.prototype.someSoundStarted = function()
{
  ;
  if(this.bStoryRunning)
  {
    this.bSoundPlaybackPhase = true;
  }
}
DialogStoryManager.prototype.recognitionStopped = function()
{
  ;
  ;
  if(this.bRecognizeUserInputPhase && !this.bSoundPlaybackPhase && this.bStoryRunning)
  {
    this.showDefaultFeedback();
  }
  if(this.bStoryRunning && this.currentState)
  {
    this.currentState.recognitionStopped();
  }
}
DialogStoryManager.prototype.somethingRecognized = function(earData)
{
  ;
  if(this.bStoryRunning)
  {
    if(earData.getConfidenceLevel() >= this.nConfidenceLevel)
    {
      this.currentState.recognized(earData.getWordId());
    }
    else
    {
      this.showDefaultFeedback();
    }
  }
}
DialogStoryManager.prototype.stopRecognition = function()
{
  ;
  if(this.bStoryRunning)
  {
    this.bRecognizeUserInputPhase = false;
    this.speechRecognitionControlBroker.requestRecognitionStop();
  }
}
DialogStoryManager.prototype.startRecognition = function()
{
  ;
  if(this.bStoryRunning)
  {
    this.bRecognizeUserInputPhase = true;
    this.speechRecognitionControlBroker.requestRecognitionStart();
  }
}
DialogStoryManager.prototype.showDefaultFeedback = function()
{
  this.showFeedback(this.view.sFeedbackDefaultText);
}
DialogStoryManager.prototype.showFeedback = function(sFeedbackText)
{
  ;
  if(this.view.bUseFeedbackPanel && this.view.feedbackPanelDivElement)
  {
    this.view.feedbackPanelDivElement.innerHTML = sFeedbackText;
  }
  if(this.dialogStoryBroker)
  {
    this.dialogStoryBroker.showFeedback();
  }
  window.setTimeout('DialogStoryManager_getInstance(\''+this.sId+'\').clearFeedback();', (this.view.nFeedbackTimeoutSeconds * 1000));
}
DialogStoryManager.prototype.clearFeedback = function()
{
  ;
  if(this.view.bUseFeedbackPanel && this.view.feedbackPanelDivElement)
  {
    this.view.feedbackPanelDivElement.innerHTML = '';
  }
  if(this.dialogStoryBroker)
  {
    this.dialogStoryBroker.hideFeedback();
  }
}
DialogStoryManager.prototype.scrollToBottom = function()
{
  ;
  if(this.scrollBroker)
  {
    this.scrollBroker.jumpToBottom();
  }
}
DialogStoryManager.prototype.stateDone = function(state)
{
  ;
  if(this.bStoryRunning)
  {
    this.currentState = state.nextState();
    ; 
    this.currentState.run();
  }
}
DialogStoryManager.prototype.getSoundStoppedAction = function()
{
  return new ObjectMethodAction(this, 'someSoundStopped');
}
DialogStoryManager.prototype.getSoundStartedAction = function()
{
  return new ObjectMethodAction(this, 'someSoundStarted');
}
DialogStoryManager.prototype.getResetAction = function()
{
  return new ObjectMethodAction(this, 'resetStory');
}
function StoryState(manager)
{
  this.init(manager);
}
StoryState.prototype.init = function(manager)
{
  this.manager = manager;
}
StoryState.prototype.recognized = function(utteranceKey)
{
  ;
}
StoryState.prototype.recognitionStopped = function()
{
  ;
}
StoryState.prototype.continueStory = function()
{
  ;
}
StoryState.prototype.nextState = function()
{
  ;
}
StoryState.prototype.run = function()
{
  ;
}
StoryState.prototype.stop = function()
{
  ;
}
StoryStateUser.prototype             = new StoryState();
StoryStateUser.prototype.constructor = StoryStateUser;
StoryStateUser.superclass            = StoryState.prototype;
function StoryStateUser(manager, aStoryElementUser)
{
  this.init(manager, aStoryElementUser);
}
StoryStateUser.prototype.init = function(manager, aStoryElementUser)
{
  StoryStateUser.superclass.init.call(this, manager);
  this.aStoryElementUser = aStoryElementUser;
  this.storyRowUser = null
  this.selectedStoryElementUser = null;
  this.soundBroker = null;
  this.bNextState = false;
}
StoryStateUser.prototype.recognized = function(utteranceKey)
{
  ;
  var knownUtterance = false;
  for(var i in this.aStoryElementUser)
  {
    var element = this.aStoryElementUser[i];
    if(element.hasUtterance(utteranceKey))
    {
      knownUtterance = true;
      var utterance = element.getUtterance(utteranceKey);
      if(utterance.hasFeedback())
      {
        this.manager.showFeedback(utterance.getFeedback());
      }
      else
      {
        this.manager.clearFeedback();
      }
      if(utterance.isCorrect())
      {
        this.selectedStoryElementUser = element;
        this.showCurrent();
        this.manager.stateDone(this);
      }
      break;
    }
  }
  if(!knownUtterance)
  {
    this.manager.showDefaultFeedback();
  }
}
StoryStateUser.prototype.continueStory = function()
{
  ;
  ;
  if(!this.manager.bSoundPlaybackPhase)
  {
    this.bNextState = true;
    this.manager.stopRecognition();
  }
  else
  {
    this.bNextState = true;
    this.recognitionStopped();
  }
}
StoryStateUser.prototype.recognitionStopped = function()
{
  ;
  if(this.bNextState)
  {
    nRandomIndex = Math.round((((this.aStoryElementUser.length-1 + 0.49999) +0.5 ) * Math.random() ) -0.5);
    this.selectedStoryElementUser = this.aStoryElementUser[nRandomIndex];
    this.showCurrent();
    this.manager.stateDone(this);
  }
}
StoryStateUser.prototype.showAll = function()
{
  if(this.storyRowUser)
    this.storyRowUser.hide();
  this.soundBroker = new SoundBroker();
  this.storyRowUser = new DialogStoryRowUser(this.aStoryElementUser[0].sId + "_userview", this.manager.getViewConfig(), this.manager.getSoundStoppedAction());
  for(var i in this.aStoryElementUser)
  {
    var userElement = this.aStoryElementUser[i];
    var soundBroker = new SoundBroker();
    this.soundBroker.addRequestSoundStopHandle(soundBroker.getRequestSoundStopAction());
    soundBroker.addSoundStoppedHandle(this.manager.getSoundStoppedAction());
    soundBroker.addRequestSoundStartHandle(this.manager.getSoundStartedAction());
    if(userElement.getSoundUrl())
    {
      var soundPlayer = new SoundPlayer(this.manager.soundPlayerRegistryManager, userElement.getSoundUrl());
      soundPlayer.setSoundBroker(soundBroker);
    }
    this.storyRowUser.addStoryElement(userElement, soundBroker);
  }
  this.storyRowUser.show();
  this.manager.scrollToBottom();
}
StoryStateUser.prototype.showCurrent = function()
{
  if(this.storyRowUser)
    this.storyRowUser.hide();
  this.soundBroker = new SoundBroker();
  if(this.selectedStoryElementUser)
  {
    var soundBroker = new SoundBroker();
    this.soundBroker.addRequestSoundStopHandle(soundBroker.getRequestSoundStopAction());
    soundBroker.addSoundStoppedHandle(this.manager.getSoundStoppedAction());
    soundBroker.addRequestSoundStartHandle(this.manager.getSoundStartedAction());
    if(this.selectedStoryElementUser.getSoundUrl())
    {
      var soundPlayer = new SoundPlayer(this.manager.soundPlayerRegistryManager, this.selectedStoryElementUser.getSoundUrl());
      soundPlayer.setSoundBroker(soundBroker);
    }
    this.storyRowUser = new DialogStoryRowUser(this.aStoryElementUser[0].sId + "_userview", this.manager.getViewConfig(), this.manager.getSoundStoppedAction());
    this.storyRowUser.addStoryElement(this.selectedStoryElementUser, soundBroker);
    this.storyRowUser.show();
    this.manager.scrollToBottom();
  }
  else
  {
    ;
  }
}
StoryStateUser.prototype.nextState = function()
{
  ;
  this.manager.stopRecognition();
  var childElement = this.selectedStoryElementUser.getChildElementSystem();
  if(childElement)
  {
    return new StoryStateSystem(this.manager, childElement);
  }
  else
  {
    return new StoryStateFinal(this.manager);
  }
}
StoryStateUser.prototype.run = function()
{
  ;
  this.showAll();
  this.manager.startRecognition();
}
StoryStateUser.prototype.stop = function()
{
  ;
  this.manager.stopRecognition();
  if(this.soundBroker)
  {
    this.soundBroker.requestSoundStop();
  }
}
StoryStateSystem.prototype             = new StoryState();
StoryStateSystem.prototype.constructor = StoryStateSystem;
StoryStateSystem.superclass            = StoryState.prototype;
function StoryStateSystem(manager, parentStoryElementSystem)
{
  this.init(manager, parentStoryElementSystem);
}
StoryStateSystem.prototype.init = function(manager, storyElementSystem)
{
  StoryStateSystem.superclass.init.call(this, manager);
  this.storyElementSystem = storyElementSystem;
  this.storyRowSystem = null;
  this.soundBroker = null;
}
StoryStateSystem.prototype.recognized = function(utteranceKey)
{
  ;
}
StoryStateSystem.prototype.recognitionStopped = function()
{
  ;
}
StoryStateSystem.prototype.continueStory = function()
{
  ;
  if(this.soundBroker)
  {
    this.soundBroker.requestSoundStop();
  }
  else
  {
    this.stateDone();
  }
}
StoryStateSystem.prototype.stateDone = function()
{
  this.manager.stateDone(this);
}
StoryStateSystem.prototype.nextState = function()
{
  ;
  if(this.storyElementSystem.hasUserContributions())
  {
    return new StoryStateUser(this.manager, this.storyElementSystem.getUserContributions());
  }
  else
  {
    return new StoryStateFinal(this.manager);
  }
}
StoryStateSystem.prototype.run = function()
{
  ;
  this.storyRowSystem = new DialogStoryRowSystem(this.storyElementSystem.sId + "_row", this.storyElementSystem, this.manager.getViewConfig());
  this.soundBroker = new SoundBroker();
  this.soundBroker.addSoundStoppedHandle(new ObjectMethodAction(this, "stateDone"));
  this.storyRowSystem.setSoundBroker(this.soundBroker);
  if(this.storyElementSystem.getSoundUrl())
  {
    var soundPlayer = new SoundPlayer(this.manager.soundPlayerRegistryManager, this.storyElementSystem.getSoundUrl());
    soundPlayer.setSoundBroker(this.soundBroker);
  }
  this.storyRowSystem.show();
  this.manager.scrollToBottom();
  this.soundBroker.requestSoundStart();
  if(!this.storyElementSystem.hasUserContributions())
  {
    this.manager.stateDone(this);
  }
}
StoryStateSystem.prototype.stop = function()
{
  ;
  if(this.soundBroker)
  {
    this.soundBroker.requestSoundStop();
  }
}
StoryStateFinal.prototype             = new StoryState();
StoryStateFinal.prototype.constructor = StoryStateFinal;
StoryStateFinal.superclass            = StoryState.prototype;
function StoryStateFinal(manager)
{
  this.init(manager);
}
StoryStateFinal.prototype.init = function(manager, storyElementSystem)
{
  StoryStateFinal.superclass.init.call(this, manager);
}
StoryStateFinal.prototype.recognized = function(utteranceKey)
{
  ;
}
StoryStateFinal.prototype.continueStory = function()
{
  ;
}
StoryStateFinal.prototype.nextState = function()
{
  ;
}
StoryStateFinal.prototype.run = function()
{
  ;
  this.manager.stopRecognition();
  this.manager.stopStory();
}
